/*
 Copyright 2011-2018 Fulco Scherjon

This file is part of the HomininSpace modelling and simulation system.

HomininSpace is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, any version 3.
HomininSpace is distributed WITHOUT ANY WARRANTY under the GNU General Public License.
HomininSpace comes with a copy of this GNU General Public License.  If not, see <http://www.gnu.org/licenses/>.

*/

package HomininSpace.common;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.query.space.grid.GridCell;
import repast.simphony.random.RandomHelper;

import HomininSpace.HomininSpaceContextBuilder;
import HomininSpace.agents.Checkpoint;

// possible debug information channels
//Constants.DEBUG_PRINT_LEVEL
//logger.info(
//System.err.write(
//System.out.write(

public final strictfp class HSUtils {
	
	private int 			currentHomininCount = 0;
	
	static private int 		homininGroupDeathCountType0 = 0;
	static private int 		homininGroupDeathCountType1 = 0;
	static private int 		homininGroupDeathCountType2 = 0;
	static private double 	homininDeathCountType0 = 0;
	static private double 	homininDeathCountType1 = 0;
	static private double 	homininDeathCountType2 = 0;
	static private int 		homininGroupColdDeathCount = 0;
	static private int 		homininGroupBorderDeathCount = 0;
	static private int 		homininGroupIberiaBorderDeathCount = 0;
	static private int 		homininGroupSizeDeathCount = 0;
	static private int 		homininGroupFloodedDeathCount = 0;
	static private int 		homininGroupMergerDeathCount = 0;
	static private int 		homininGroupHungerDeathCount = 0;
	static private int 		homininGroupOffspring = 0;
	
	static private double 	minLongitude = 100000.0;
	static private double 	maxLongitude = -100000.0;
	static private double 	minLatitude = 100000.0;
	static private double 	maxLatitude = -100000.0;
	
    static int 				latestIndex = -1;
    static public int 		startingTimeStep = -1;
    static public int 		timeStepIncrement = 1;
    static public int 		numberOftimeSteps = 0;


    public static void initialize()
    {
    	homininGroupDeathCountType0 = 0;
    	homininGroupDeathCountType1 = 0;
    	homininGroupDeathCountType2 = 0;
    	homininDeathCountType0 = 0;
    	homininDeathCountType1 = 0;
    	homininDeathCountType2 = 0;
    	homininGroupColdDeathCount = 0;
    	homininGroupBorderDeathCount = 0;
    	homininGroupIberiaBorderDeathCount = 0;
    	homininGroupSizeDeathCount = 0;
    	homininGroupFloodedDeathCount = 0;
    	homininGroupMergerDeathCount = 0;
    	homininGroupHungerDeathCount = 0;
    	homininGroupOffspring = 0;
    	
    	minLongitude = 100000.0;
    	maxLongitude = -100000.0;
    	minLatitude = 100000.0;
    	maxLatitude = -100000.0;
    	
        latestIndex = -1;
        startingTimeStep = -1;
        timeStepIncrement = 1;
        numberOftimeSteps = 0;
    }
    
	public int getCurrentHomininCount()
	{
		return currentHomininCount;
	}
	public void setCurrentHomininCount( int count )
	{
		currentHomininCount = count;
	}
	
	public static int getHomininGroupDeathCountPerCause(int cause)
	{
		int res = -1;
		switch(cause) {
		case Constants.CAUSE_OF_DEATH_BORDER:
			res = homininGroupBorderDeathCount;
			break;
		case Constants.CAUSE_OF_DEATH_BORDER_IBERIA:
			res = homininGroupIberiaBorderDeathCount;
			break;
		case Constants.CAUSE_OF_DEATH_COLD:
			res = homininGroupColdDeathCount;
			break;
		case Constants.CAUSE_OF_DEATH_FLOODED:
			res = homininGroupFloodedDeathCount;
			break;
		case Constants.CAUSE_OF_DEATH_SIZE:
			res = homininGroupSizeDeathCount;
			break;
		case Constants.CAUSE_OF_DEATH_MERGER:
			res = homininGroupMergerDeathCount;
			break;
		case Constants.CAUSE_OF_DEATH_HUNGER:
			res = homininGroupHungerDeathCount;
			break;
		default:
			break;
		}

		return res;
	}
	
	public static int getHomininGroupDeathCount(int type)
	{
		int res = -1;
		if (type == 0)
			res = homininGroupDeathCountType0;
		else
			if (type == 1)
				res = homininGroupDeathCountType1;
			else
				res = homininGroupDeathCountType2;
		return res;
	}
	public static double getHomininDeathCount(int type)
	{
		double res = 0;
		if (type == 0)
			res = homininDeathCountType0;
		else
			if (type == 1)
				res = homininDeathCountType1;
			else
				res = homininDeathCountType2;
		return Math.round(res);
	}
	public static int getHomininOffspring(  )
	{
		return homininGroupOffspring;
	}

	public static void incrementHomininGroupDeathCount( int type, double size, int cause )
	{
		incrementHomininDeathCount( type, size);
		
		if (type == 0)
		{
			homininGroupDeathCountType0++;
		}
		else
			if (type == 1)
			{
				homininGroupDeathCountType1++;
			}
			else
			{
				homininGroupDeathCountType2++;
			}
		
		switch(cause) {
		case Constants.CAUSE_OF_DEATH_BORDER:
			homininGroupBorderDeathCount++;
			break;
		case Constants.CAUSE_OF_DEATH_BORDER_IBERIA:
			homininGroupIberiaBorderDeathCount++;
			break;
		case Constants.CAUSE_OF_DEATH_COLD:
			homininGroupColdDeathCount++;
			break;
		case Constants.CAUSE_OF_DEATH_FLOODED:
			homininGroupFloodedDeathCount++;
			break;
		case Constants.CAUSE_OF_DEATH_SIZE:
			homininGroupSizeDeathCount++;
			break;
		case Constants.CAUSE_OF_DEATH_MERGER:
			homininGroupMergerDeathCount++;
			break;
		case Constants.CAUSE_OF_DEATH_HUNGER:
			homininGroupHungerDeathCount++;
			break;
		default:
			break;
		}
	}
	
	public static void incrementHomininDeathCount( int type, double size )
	{
		if (type == 0)
		{
			homininDeathCountType0 += size;
		}
		else
			if (type == 1)
			{
				homininDeathCountType1 += size;
			}
			else
			{
				homininDeathCountType2 += size;
			}
	}
	public static void incrementHomininOffspring(  )
	{
		homininGroupOffspring++;
	}
	
	/*
	 * Returns a random element of the specified arbitrary list.
	 */
	public static <T> T randomElementOf(final List<T> list) {
	    		if (null == list) {
	    			throw new IllegalArgumentException("Parameter list cannot be null.");
	    		}

	    		if (list.isEmpty()) {
	    			throw new IllegalArgumentException(
	    					"Cannot return a random element from an empty list.");
	    		}

	    		return list.get(RandomHelper.nextIntFromTo(0, list.size() - 1));
    }

	    	/*
	    	 * Returns a filtered list of {@link GridCell} instances from the specified
	    	 * collection which holds no objects.
	    	 * 
	    	 * @param neighborhood
	    	 *            an arbitrary list of <code>GridCell</code> objects; <i>cannot
	    	 *            be <code>null</code></i>
	    	 * @return a filtered list of empty <code>GridCell</code> objects, more
	    	 *         specifically for which <code>size()</code> returns <code>0</code>
	    	 *         ; if there is no empty cell in the specified list, an empty list
	    	 *         is returned
	    	 */
    public static <T> List<GridCell<T>> getFreeGridCells(
                        final List<GridCell<T>> neighborhood) {
    		if (null == neighborhood) {
    			throw new IllegalArgumentException(
    					"Parameter neighborhood cannot be null.");
    		}

    		final ArrayList<GridCell<T>> ret = new ArrayList<GridCell<T>>();

    		for (final GridCell<T> act : neighborhood) {
    			if (0 == act.size()) {   // size is number of agents at this gridcell
    				ret.add(act);
    			/*
    		        for (final Object object : act.items()) {
    	                if (object instanceof HabitatCell) {
    	                    HabitatCell h = (HabitatCell) object;
    	                    if (h.getCapacity() != Constants.BELOW_SEALEVEL)
    	                    {
    	                    	ret.add(act);
        	        			throw new IllegalArgumentException("found.");
    	                    }

    	                }
    		        }
    		        */    				
    			}
    		}

    		return ret;
    }
/*        
    	public double distance(Coordinate p0, Coordinate p1) 
    	{
    		Geometry p0Geom = this.geomFac.createPoint(p0);
    		Geometry p1Geom = this.geomFac.createPoint(p1);
    		return convertToMeters( DistanceOp.distance( p0Geom, p1Geom) ); 
    	}
*/
        /*
         * Read input file exported from SAM
         * 
Longitude (X Centroid)	Latitude (Y Centroid)	ID	LONG	        LAT	    n_bio12	Mean_bio12	n_alt	Mean_alt	n_bio1	Mean_bio1
-4.071	                41.629	                15	-40710000000	41629	4	    452.250	    4	    832.500	    4	    117
-3.971	41.629	16	-39710000000	41629	6	453.667	6	824.333	6	117

The X coordinate steps 0.1 degrees, the Y coordinate steps 0.1 degrees

 The loop starts bottom left on the map, and does not present empty cells. Thus it does not start at x-min, y-max!
 
 First read the whole file to determine x-min, x-max, y-min, y-max
         */
        
public static List<CellData> readClimateDataFile(final String cellDataFileName) {
        	
        	Constants.DEBUG_PRINT_LEVEL("Reading climate data file: " + cellDataFileName, 40);
        	
            final ArrayList<CellData> ret = new ArrayList<CellData>();
            BufferedReader br = null;
            String line = null;
            int lineCount = 0;
            double minCurrentTemperature = 1000;
            double maxCurrentTemperature = 0;
            double minLGMTemperature = 1000;
            double maxLGMTemperature = 0;
            double minPrecipitation = 1000;
            double maxPrecipitation = 0;

            try {
                br = new BufferedReader(new FileReader(cellDataFileName));

                // Read file, parse longitude and latitude to find minimum and maximum
                minLongitude = 100000.0;
                maxLongitude = -100000.0;
                minLatitude = 100000.0;
                maxLatitude = -100000.0;

                // Skip header
                br.readLine();
                
                while ((line = br.readLine()) != null) 
                {
                        // Split the line around whitespaces
                        final String[] data = line.split("\\s+");

                        int idx = 0;
                        final double longXC = Double.parseDouble(data[idx++]);
                        final double latYC = Double.parseDouble(data[idx++]);

                        if (longXC < minLongitude) minLongitude = longXC;
                        if (longXC > maxLongitude) maxLongitude = longXC;
                        if (latYC < minLatitude) minLatitude = latYC;
                        if (latYC > maxLatitude) maxLatitude = latYC;
                        lineCount++;
                }
                
                Constants.DEBUG_PRINT_LEVEL("Read climate input file, minX=" + minLongitude + 
                		" maxX=" + maxLongitude + " minY=" + minLatitude + " maxY=" + maxLatitude + " lines:" + lineCount, 5);

                /*
                 * read again, now assigning climate variables to grid positions
                 */
                br = new BufferedReader(new FileReader(cellDataFileName));

                // Skip header
                br.readLine();

                // Read lines, parse data and add a new CellData for each one
                int x = 0;
                int y = 0;
                int seaCellsCount = 0;
                    
/*
BIO1 = Annual Mean Temperature
BIO2 = Mean Diurnal Range (Mean of monthly (max temp - min temp))
BIO3 = Isothermality (BIO2/BIO7) (* 100)
BIO4 = Temperature Seasonality (standard deviation *100)
BIO5 = Max Temperature of Warmest Month
BIO6 = Min Temperature of Coldest Month
BIO7 = Temperature Annual Range (BIO5-BIO6)
BIO8 = Mean Temperature of Wettest Quarter
BIO9 = Mean Temperature of Driest Quarter
BIO10 = Mean Temperature of Warmest Quarter
BIO11 = Mean Temperature of Coldest Quarter
BIO12 = Annual Precipitation
BIO13 = Precipitation of Wettest Month
BIO14 = Precipitation of Driest Month
BIO15 = Precipitation Seasonality (Coefficient of Variation)
BIO16 = Precipitation of Wettest Quarter
BIO17 = Precipitation of Driest Quarter
BIO18 = Precipitation of Warmest Quarter
BIO19 = Precipitation of Coldest Quarter
*/
                while ((line = br.readLine()) != null) 
                {
                   	CellData newCell;
                    int idx = 0;
                    
                    // Split the line around whitespaces
                    final String[] data = line.split("\\s+");

                    final double longXC = Double.parseDouble(data[idx++]);
                    final double latYC = Double.parseDouble(data[idx++]);
                    final int n_alt = Integer.parseInt(data[idx++]);
                    // if n = 0 then the average is from 0 points, no average thus, means sea
                    final double mean_alt = Double.parseDouble(data[idx++]);
                    final double mean_bio1 = Double.parseDouble(data[idx++]);
                    final double mean_bio5 = Double.parseDouble(data[idx++]);
                    final double mean_bio6 = Double.parseDouble(data[idx++]);
                    final double mean_bio12 = Double.parseDouble(data[idx++]);
                    final double mean_bio13 = Double.parseDouble(data[idx++]);
                    final double mean_bio14 = Double.parseDouble(data[idx++]);
                    final double mean_CCSM_bio14 = Double.parseDouble(data[idx++]);
                    final double mean_CCSM_bio1 = Double.parseDouble(data[idx++]);
                    final double mean_CCSM_bio5 = Double.parseDouble(data[idx++]);
                    final double mean_CCSM_bio6 = Double.parseDouble(data[idx++]);
                    final double mean_CCSM_bio12 = Double.parseDouble(data[idx++]);
                    final double mean_CCSM_bio13 = Double.parseDouble(data[idx++]);
                    // ignore 13 and 14 for now
                    final double max_bedrock = Double.parseDouble(data[idx++]);

                    x = ConvertLongitude2Grid( longXC );
                    y = ConvertLatitude2Grid( latYC );
                            
                    //Constants.DEBUG_PRINT_LEVEL("Longitude: " + longXC + "  x=" + x, 100);
                    //Constants.DEBUG_PRINT_LEVEL("Latitude: " + latYC + "  y=" + y, 100);
                            
                    newCell = new CellData(x, y);
                    newCell.setHeight(mean_alt);
                    newCell.setDepth(max_bedrock);
                    
                   	newCell.setType(Constants.HABITAT_TYPE_PLAIN);
                    if ((n_alt == 0) || (mean_alt<0.5))
                    {
                       	//Constants.DEBUG_PRINT_LEVEL("  x=" + x + "  y=" + y + " n_alt: " + n_alt + "  mean_alt=" + mean_alt, 100);
                        if (n_alt == 0)
                        {
                       	    newCell.setType(Constants.HABITAT_TYPE_SEA);
                       	    seaCellsCount++;
//                       	  newCell.setHeight(0.0);
                       	    newCell.setHeight(max_bedrock);
                        }
                        else
                        {
                        	// this assumes that lakes have more than 0 measuring points and still an height less than 1
                        	// it also assumes that coastal areas have no height less than 1
                       	    newCell.setType(Constants.HABITAT_TYPE_LAKE);
                        }
                    }

                    newCell.setCurrentTemperature(mean_bio1/10);    // Temperature = 10 * C in the input file
                    newCell.setCurrentWarmestMonthTemperature(mean_bio5/10);    // Temperature = 10 * C in the input file
                    newCell.setCurrentColdestMonthTemperature(mean_bio6/10);    // Temperature = 10 * C in the input file
                    newCell.setCurrentPrecipitation(mean_bio12);
                    newCell.setLGMTemperature(mean_CCSM_bio1/10);    // Temperature = 10 * C in the input file
                    newCell.setLGMPrecipitation(mean_CCSM_bio12);
                    newCell.setLGMWarmestMonthTemperature(mean_CCSM_bio5/10);    // Temperature = 10 * C in the input file
                    newCell.setLGMColdestMonthTemperature(mean_CCSM_bio6/10);    // Temperature = 10 * C in the input file

                    if (minCurrentTemperature > mean_bio6/10) minCurrentTemperature = mean_bio6/10;
                    if (maxCurrentTemperature < mean_bio5/10) maxCurrentTemperature = mean_bio5/10;
                    if (minLGMTemperature > mean_CCSM_bio6/10) minLGMTemperature = mean_CCSM_bio6/10;
                    if (maxLGMTemperature < mean_CCSM_bio5/10) maxLGMTemperature = mean_CCSM_bio5/10;
                    if ((minPrecipitation > mean_bio12) && (mean_bio12 > 0)) minPrecipitation = mean_bio12;
                    if (maxPrecipitation < mean_bio12) maxPrecipitation = mean_bio12;
                            
//                    Constants.DEBUG_PRINT_LEVEL(" cur precip: " + mean_bio12 + "  LGM precip: " + mean_CCSM_bio12, 5);
                    ret.add(newCell);
                }
                    
                Constants.DEBUG_PRINT_LEVEL("Read climate input file, x=" + x + " y=" + y + " sea cells:" + seaCellsCount, 5);
                Constants.DEBUG_PRINT_LEVEL("LGM minTemperature=" + minLGMTemperature + " maxTemperature=" + maxLGMTemperature, 5);
                Constants.DEBUG_PRINT_LEVEL("Current minTemperature=" + minCurrentTemperature + " maxTemperature=" + maxCurrentTemperature, 5);
                Constants.DEBUG_PRINT_LEVEL("Current minPrecipitation=" + minPrecipitation + " maxPrecipitation=" + maxPrecipitation, 5);

            } catch (final FileNotFoundException e) {
                    e.printStackTrace();
            } catch (final IOException e) {
                    e.printStackTrace();
            } finally {
                    if (br != null) {
                            try {
                                    br.close();
                            } catch (final IOException e) {
                                    e.printStackTrace();
                            }
                    }
            }

            return Collections.unmodifiableList(ret);
}
 

/*
 * Read file with checkpoints    
 * 		There are four types of checkpoints:
 * 		- general: represents an archaeological site with timed intervals of presence and/or absence
 * 		- monitoring: will register if agents are in the area but has no intervals to compare against
 * 		- starting points: where initial populations can be placed
 * 		- climate: will register climate change around the checkpoint
 */
public static List<Checkpoint> readCheckpointDataFile(final String cellDataFileName) 
{
    	Constants.DEBUG_PRINT_LEVEL("Reading checkpoint data file: " + cellDataFileName, 10);
    	
        final ArrayList<Checkpoint> ret = new ArrayList<Checkpoint>();
        BufferedReader 				br = null;
        String 						line = null;
        int 						lineCount = 0;
        String 						previousName = null;
        Checkpoint 					newCheckpoint = null;
        int 						CSTCount = 0;
        
        try {
            br = new BufferedReader(new FileReader(cellDataFileName));

            // Skip header
            br.readLine();

            while ((line = br.readLine()) != null) 
            {
                // Split the line around tabs
                final String[] data = line.split("\\t+");
                int idx = 0;
                final int pointType = Integer.parseInt(data[idx++]);
                final String name = data[idx++];
                
                Constants.CheckpointType checkpointType = Constants.CheckpointType.CHECKPOINT_TYPE_GENERAL_CST;
                if (pointType == 2) checkpointType = Constants.CheckpointType.CHECKPOINT_TYPE_MONITORING_CST;  
                if (pointType == 3) checkpointType = Constants.CheckpointType.CHECKPOINT_TYPE_STARTINGPOINT_CST;  
                if (pointType == 4) checkpointType = Constants.CheckpointType.CHECKPOINT_TYPE_CLIMATE_CST;  
                
                Constants.DEBUG_PRINT_LEVEL("Checkpoint <" + name + ">  number:" + CSTCount, 40);
                
               	final double latitude = Double.parseDouble(data[idx++]);
               	final double longitude = Double.parseDouble(data[idx++]);
                
                if (!name.equals(previousName) || (newCheckpoint == null))
                {
                	previousName = name;
                	
                	newCheckpoint = new Checkpoint(CSTCount++, checkpointType);
                    ret.add(newCheckpoint);
                    newCheckpoint.setName(name);
                    Constants.DEBUG_PRINT_LEVEL("new Checkpoint <" + name + ">", 40);
                    
                    final int yc = ConvertLatitude2Grid(latitude);
                    final int xc = ConvertLongitude2Grid(longitude);
                    newCheckpoint.setPosX(xc);
                    newCheckpoint.setPosY(yc);
                }
                
                if (checkpointType == Constants.CheckpointType.CHECKPOINT_TYPE_GENERAL_CST)
                {
	                final double age = Double.parseDouble(data[idx++]);
	                final double plus = Double.parseDouble(data[idx++]);
	                final double minus = Double.parseDouble(data[idx++]);
	                final int confidenceLevel = Integer.parseInt(data[idx++]);
	
	                int endAge = (int) Math.round(age * 1000 - minus * 1000);  // ages in kya
	                int startAge = (int) Math.round(age * 1000 + plus * 1000);
	                Constants.DEBUG_PRINT_LEVEL("start: " + startAge + " end: " + endAge, 40);
	                
	                newCheckpoint.addPresenceInterval(startAge, endAge, confidenceLevel);
                }
                
                lineCount++;
            }
            
            Constants.DEBUG_PRINT_LEVEL("Read checkpoint input file, #lines:" + lineCount + " #cst:" + CSTCount, 10);
 
        } catch (final FileNotFoundException e) {
                e.printStackTrace();
        } catch (final IOException e) {
                e.printStackTrace();
        } finally {
                if (br != null) {
                        try {
                                br.close();
                        } catch (final IOException e) {
                                e.printStackTrace();
                        }
                }
        }

        return Collections.unmodifiableList(ret);
}

    /*
     * Read sealevel file from Bintanja 2008

Bintanja and van de Wal 2008 Global Temperature, Sea Level, and Ice Volume Reconstructions						

Column								
1. Time (kyr BP)								
2. Modelled marine oxygen isotope value, relative to present (o/oo)								
3. Ice sheet contribution to the marine isotope signal, relative to present (o/oo)							
4. Deep-ocean temperature contribution to the marine isotope signal, relative to present (o/oo)						
5. Atmospheric surface air temperature relative to present (degC)								
6. Deep-ocean temperature relative to present (degC)								
7. Eurasian ice volume relative to present (m sea level equivalent)								
8. North American ice volume relative to present (m sea level equivalent)								
9. Global sea level relative to present (m)								
								
Time	Iso_tot	Iso_ice	Iso_do	Tsurf	Tdo	Ice_eas	Ice_nam	RSL
0.00000	-0.0029	0.0102	-0.0131	-0.0479	0.0570	1.164	0.1107	1.459
0.10010	-0.0035	0.0104	-0.0139	-0.0614	0.0604	1.207	0.1091	1.507
0.20000	-0.0039	0.0108	-0.0147	-0.0665	0.0638	1.238	0.1078	1.541
0.30000	-0.0043	0.0111	-0.0154	-0.0707	0.0670	1.281	0.1101	1.593

     */
    
    static double timeStamps[];
    static double seaLevels[];
    static double temperatures[];
    static int levelCount = 0;
    
    public static void readBintanjeDataFile(final String cellDataFileName) {

    	timeStamps = new double[1500];
    	seaLevels = new double[1500];
    	temperatures = new double[1500];
    	
    	double		minSeaLevel = 0;
    	double		maxSeaLevel = 0;
    	double		minTemperature = 0;
    	double		maxTemperature = 0;
    	
    	Constants.DEBUG_PRINT_LEVEL("Reading Bintanja file: " + cellDataFileName, 40);
    	
        BufferedReader br = null;
        String line = null;
        levelCount = 0;

        try {
            br = new BufferedReader(new FileReader(cellDataFileName));

            // Skip header 17 x
            br.readLine(); br.readLine(); br.readLine(); br.readLine(); br.readLine();
            br.readLine(); br.readLine(); br.readLine(); br.readLine(); br.readLine();
            br.readLine(); br.readLine(); br.readLine(); br.readLine(); br.readLine();
            br.readLine(); br.readLine(); 
            
            /*
             * read values, select needed ones from interval
             */
            while ((line = br.readLine()) != null) 
            {
            	 // Split the line around whitespaces
                 final String[] data = line.split("\\s+");
                 int idx = 0;
                 //Time	Iso_tot	Iso_ice	Iso_do	Tsurf	Tdo	Ice_eas	Ice_nam	RSL

                 final double time = Double.parseDouble(data[idx++]);
                 Double.parseDouble(data[idx++]);  // Iso_tot
                 Double.parseDouble(data[idx++]);  // Iso_ice
                 Double.parseDouble(data[idx++]);  // Iso_do
                 final double Tsurf = Double.parseDouble(data[idx++]);
                 
                 Double.parseDouble(data[idx++]);  // Tdo
                 Double.parseDouble(data[idx++]);  // Ice_eas
                 Double.parseDouble(data[idx++]);  // Ice_nam
                 final double RSL = Double.parseDouble(data[idx++]);
                 
                 // ensure all dates fall within simulation period and allow for extrapolation
                 if ((time>(Constants.END_TIME_STEP-1)) && (time<(Constants.START_TIME_STEP+1)))
                 {
                	 levelCount++;
                	 Constants.DEBUG_PRINT_LEVEL("Time: " + time + " linecount " + levelCount, 100);
                	                      
                	 timeStamps[levelCount] = time;
                	 seaLevels[levelCount] = RSL;
                	 temperatures[levelCount] = Tsurf;
                	 
                	 if (minSeaLevel > RSL) minSeaLevel = RSL;
                	 if (maxSeaLevel < RSL) maxSeaLevel = RSL;
                	 if (minTemperature > Tsurf) minTemperature = Tsurf;
                	 if (maxTemperature < Tsurf) maxTemperature = Tsurf;
                	 
                 }
             }
                
             Constants.DEBUG_PRINT_LEVEL("Read Bintanja input file, lines=" + levelCount + " (max = 1500!)", 10);
             Constants.DEBUG_PRINT_LEVEL("MinSeaLevel=" + minSeaLevel + " MaxSeaLevel=" + maxSeaLevel, 10);
             Constants.DEBUG_PRINT_LEVEL("MinTemperature=" + minTemperature + " MaxTemperature=" + maxTemperature, 10);

        } catch (final FileNotFoundException e) {
                e.printStackTrace();
        } catch (final IOException e) {
                e.printStackTrace();
        } finally {
                if (br != null) {
                        try {
                                br.close();
                        } catch (final IOException e) {
                                e.printStackTrace();
                        }
                }
        }
    }
    
	static public double getSeaLevel( int tickcount )
	{
		int currentTime = startingTimeStep - (tickcount * timeStepIncrement);
		
		// find index for the current time stamp
		if (latestIndex == -1)
		{
			latestIndex = levelCount;
		}
		
		// match with closest timestamp
		while ( currentTime < Math.round(timeStamps[latestIndex]*1000) )
		{
			latestIndex--;
		}
		
		return seaLevels[latestIndex];
	}
	

	static public double getTemperature( int tickcount )
	{
		int currentTime = startingTimeStep - (tickcount * timeStepIncrement);
		
		// find index for the current time stamp
		if (latestIndex == -1)
		{
			latestIndex = levelCount;
		}
		
		while ( currentTime < Math.round(timeStamps[latestIndex]*1000) )
		{
			latestIndex--;
			
		}
		
		return temperatures[latestIndex];
	}

	/*
	 * Find temperature offset for current tick count
	 */
	static public double getTemperatureOffset( )
	{
		ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
		int tickcount =  (int)Math.round(schedule.getTickCount());
		
		int currentTime = startingTimeStep - (tickcount * timeStepIncrement);
		
		// find index for the current time stamp
		if (latestIndex == -1)
		{
			latestIndex = levelCount;
		}
		
		while ( currentTime < Math.round(timeStamps[latestIndex]*1000) )
		{
			latestIndex--;
		}
		
		return temperatures[latestIndex];
	}
	
	
    static public int ConvertLatitude2Grid( double latitude )
    {
    	return (int) Math.round(latitude * 10 - minLatitude * 10);    	
    }
    
    static public int ConvertLongitude2Grid( double longitude )
    {
    	int offset = (int) (Math.round( minLongitude * 10 ) * -1);
    	return (int) Math.round( longitude * 10 ) + offset;
    }
    
	static public int getVisualValueStyle()
	{
		//ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
		//int tickcount =  (int)Math.round(schedule.getTickCount());
		
		double tickcount = HomininSpaceContextBuilder.getTickCount();
		
		// first few turns show topography
		if (tickcount <= Constants.VISIBLE_LAYER_TOPOGRAPHY_TURN_COUNT)
		{
			return Constants.VISIBLE_LAYER_TOPOGRAPHY;
		}
		else if (HomininSpaceContextBuilder.getSimulationHasEnded())
		{
			return Constants.VISIBLE_LAYER_AT_END;
		}
		else
		{
			return Constants.VISIBLE_LAYER_WHILE_RUNNING;
		}
	}
    
	static public String getPercentageString( double count, double total)
	{
		String s;
		double percent = 0;
		
		if (total>0)
		{
			percent = (count/total) * 100;
		}
		
		s = "" + Math.round(percent*100)/100;
		return s + "%";
	}
    /*
     * Hidden constructor to ensure no instances are created.
     */
    private HSUtils() {
    		;
    }
        
}

