/*
 Copyright 2011-2018 Fulco Scherjon

This file is part of the HomininSpace modelling and simulation system.

HomininSpace is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, any version 3.
HomininSpace is distributed WITHOUT ANY WARRANTY under the GNU General Public License.
HomininSpace comes with a copy of this GNU General Public License.  If not, see <http://www.gnu.org/licenses/>.

*/

package HomininSpace.observer;

import java.awt.Color;

import repast.simphony.valueLayer.ValueLayer;
import repast.simphony.visualizationOGL2D.ValueLayerStyleOGL;

import HomininSpace.common.Constants;
import HomininSpace.common.HSUtils;

/*
 * VisibleValueLayerStyleOGL, colouring of the grid cells on visual display
 */
public class VisibleValueLayerStyleOGL implements ValueLayerStyleOGL {
    private ValueLayer layer = null;

	@Override
	public Color getColor(double... coordinates) {
		double value = layer.get(coordinates);
		int redValue = 0;
		int greenValue = 0;
		int blueValue = 255;
		int visibleType = HSUtils.getVisualValueStyle();
		
		// what values should be displayed?
		if (visibleType == Constants.VISIBLE_LAYER_TOPOGRAPHY)
		{
			if (value < 1.0 )
			{
				blueValue = 255;
				redValue = 0;
				greenValue = 0;
			}
			else
			{
				redValue = 50;
				blueValue = 0;
				
				// color of the grid cells
				value = Math.max(value, 10);
				
				if (value > 1000) blueValue = 200;
				else
					if (value > 800) blueValue = 150;
					else
						if (value > 600) blueValue = 100;
						else
							if (value > 300) blueValue = 50;
				
				greenValue = (int) Math.min( (value / 1500) * 255, 255);  // was 700
			}
		}
		else if (visibleType == Constants.VISIBLE_LAYER_CLIMATE)
		{
			// values in Valuelayer are biome:
			if (value == Constants.CLIMATE_TYPE_POLAR)
			{   // wit
				blueValue = 255;
				redValue = 255;
				greenValue = 255;
			}
			if (value == Constants.CLIMATE_TYPE_BOREAL)
			{	// blue
				blueValue = 255;
				redValue = 0;
				greenValue = 200;
			}
			if (value == Constants.CLIMATE_TYPE_COOL_GRASS)
			{	// blue/green?
				blueValue = 66;
				redValue = 0;
				greenValue = 155;
			}
			if (value == Constants.CLIMATE_TYPE_COOL_FOREST)
			{	// blue/green?
				blueValue = 100;
				redValue = 28;
				greenValue = 128;
			}
			if (value == Constants.CLIMATE_TYPE_WARM_GRASS)
			{	// green
				blueValue = 4;
				redValue = 208;
				greenValue = 225;
			}
			if (value == Constants.CLIMATE_TYPE_WARM_FOREST)
			{	// green
				blueValue = 3;
				redValue = 133;
				greenValue = 143;
			}
			if (value == Constants.CLIMATE_TYPE_SUBTROPICAL)
			{   // warm green?
				blueValue = 0;
				redValue = 145;
				greenValue = 255;
			}
			if (value == Constants.CLIMATE_TYPE_TROPICAL)
			{	// oranje
				blueValue = 17;
				redValue = 238;
				greenValue = 145;
			}
			if (value == Constants.CLIMATE_TYPE_EQUATORIAL)
			{	// red
				blueValue = 0;
				redValue = 255;
				greenValue = 0;
			}
		} else if (visibleType == Constants.VISIBLE_LAYER_ENERGY)
		{
			//Constants.DEBUG_PRINT_LEVEL("Value " + value , 30 );
			if (value < 1.0 )
			{
				blueValue = 255;
				redValue = 255;
				greenValue = 255;
			}
			else
			{
				value = Math.min(value, 255);
				redValue = 10;
				greenValue = 10;
				blueValue = (int) value;
			}
			
		} else if (visibleType == Constants.VISIBLE_LAYER_VISITED)
		{	
			// Density map of visits
			// 		only created at the end of the simulation, not each turn
			if (value < 1.0 )
			{
				blueValue = 255;
				redValue = 255;
				greenValue = 255;
			}
			else
			{
				blueValue = 10;
				
				value = Math.min(value, 150000);
				double rat = ((value)/150000.0) * 255;
				redValue = (int) (255 - ( Math.round(rat) + 1));
/*				
				redValue = 255;
				if (value > 100) redValue = 240;
				if (value > 500) redValue = 220;
				if (value > 1000) redValue = 200;
				if (value > 5000) redValue = 190;
				if (value > 10000) redValue = 180;
				if (value > 15000) redValue = 170;
				if (value > 20000) redValue = 160;
				if (value > 30000) redValue = 150;
				if (value > 40000) redValue = 140;
				if (value > 50000) redValue = 130;
				if (value > 55000) redValue = 120;
				if (value > 60000) redValue = 110;
				if (value > 65000) redValue = 100;
				if (value > 70000) redValue = 90;
				if (value > 75000) redValue = 80;
				if (value > 80000) redValue = 70;
				if (value > 85000) redValue = 60;
				if (value > 90000) redValue = 50;
				if (value > 100000) redValue = 40;
				if (value > 150000) redValue = 30;
				if (value > 200000) redValue = 20;
				if (value > 250000) redValue = 10;
				if (value > 300000) redValue = 5;
				if (value > 350000) redValue = 1;
				
*/
			}
		} else if (visibleType == Constants.VISIBLE_LAYER_DEAD)
		{   
			// Density map of number of deads
			// 		only called at the end of the simulation, not each turn
			if (value < 1.0 )
			{
				blueValue = 255;
				redValue = 255;
				greenValue = 255;
			}
			else
			{
				redValue = 10;
				
				blueValue = (int) (255 - value * 4);
				if (value > 10) blueValue = (int) (200 - value+10);
				if (value > 50) blueValue = (int) (175 - value/2);
				if (value > 100) blueValue = 150;
				if (value > 500) blueValue = 125;
				if (value > 1000) blueValue = 100;
				if (value > 1500) blueValue = 75;
				if (value > 2000) blueValue = 50;
				if (value > 3000) blueValue = 40;
				if (value > 4000) blueValue = 30;
				if (value > 5000) blueValue = 20;
				if (value > 7500) blueValue = 15;
				if (value > 10000) blueValue = 10;
				if (value > 20000) blueValue = 5;
			}
		}
		
        return new Color(redValue, greenValue, blueValue);
	}

	@Override
	public float getCellSize() {
        return 15.0f;
	}

	@Override
	public void init(ValueLayer layer) {
        this.layer = layer;
	}
}


